/*-------------------------------------------------------------
 *  Copyright (c) 2011, MSS ComBlock. All rights reserved.
 *
 * 	Filename: RF.c
 *  Last modified: 6/13/11
 *  Authors:
 *  	Alain Zarembowitch / MSS ComBlock
 *  Version: 1.1
 *  Inheritance: N/A
 *  Description: RF components driver
 *
 -------------------------------------------------------------*/
#include "main.h"
#include "utils.h"
#include "rf.h"

// global
int pd1, pd2;	// remember power down condition for each transceiver
int freqBand1, freqBand2;	// remember frequency band for each transceiver 0 = 2.4GHz, 1 = 5GHz
int rx_txn1, rx_txn2;	// detect changes in RX_TXn
unsigned long adc_mux;	// adc mux selection. range 0 - 7
unsigned long rx1_rssi;	// read via AD0[5]
unsigned long rx2_rssi;	// read via AD0[5]
unsigned long rx1_agc;	// read via AD0[5]
unsigned long rx2_agc;	// read via AD0[5]
unsigned long tx1_gain_ctrl;	// read via AD0[5]
unsigned long tx2_gain_ctrl;	// read via AD0[5]
unsigned long tx1_power_det;	// read via AD0[5]
unsigned long tx2_power_det;	// read via AD0[5]
const unsigned char gainTable5GHz[96] =
	{0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17,18,18,19,19,
	20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27,28,28,29,29,30,30,31,31,
	88,89,89,90,90,91,91,92,92,93,93,94,94,95,95,
	119,120,120,121,121,122,122,123,123,124,124,125,125,126,126,127,127};
const unsigned char gainTable2p4GHz[96] =
	{0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17,18,18,19,19,
	20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27,28,28,29,29,30,30,31,
	88,89,89,90,90,91,91,92,92,93,93,94,94,95,
	118,119,119,120,120,121,121,122,122,123,123,124,124,125,125,126,126,127,127};

void max2829_init(int ICno) {

	if (ICno == 1) { // Transceiver 1
		// initialization
		// SPI reset
		// "It is recommended to reset the SPI and all registers at the start of power-up
		// to ensure that the registers are set to the correct values"
		PIN_XCVR1_SHDNB_L();
		PIN_XCVR1_RXENA_H();
		PIN_XCVR1_TXENA_H();

		if (regv[0] & 0x80) {
			// transceiver 1 power down
			PIN_XCVR1_SHDNB_L();
			PIN_XCVR1_RXENA_L();
			PIN_XCVR1_TXENA_L();
			pd1 = 1; // keep track of power down
		} else {
			// transceiver 1 in standby:  no tx/rx, just synthesizer
			PIN_XCVR1_RXENA_L();
			PIN_XCVR1_TXENA_L();
			PIN_XCVR1_SHDNB_H();
			pd1 = 0;

			// Standby register 2
			//	max2829_reg(0x3007,2,1);	// MIMO applications????
			max2829_reg(0x1007, 2, 1); // Normal operation
			// frequency
			program_tuner_pll(1);
			// bandwidths
			bandwidth(1);
			// Rx Control/RSSI Register
			// Rx VGA gain programmed with serial data bits in the Rx gain register (D6:D0).
			// RSSI range 0.5-2.5V
			// RSSI always enabled, 100 Hz AC coupling -3dB cutoff frequency
			max2829_reg(0x1C21, 8, 1);
			// Tx Linearity/Baseband Gain Register
			// Enable Tx VGA Gain Programming Serially
			// maximum Tx VGA Linearity
			// maximum Tx Upconverter Linearity
			// maximum Tx Baseband Gain
			max2829_reg(0x04CF, 9, 1);

		}
	} else if (ICno == 2) {
		// initialization
		// SPI reset
		// "It is recommended to reset the SPI and all registers at the start of power-up
		// to ensure that the registers are set to the correct values"
		PIN_XCVR2_SHDNB_L();
		PIN_XCVR2_RXENA_H();
		PIN_XCVR2_TXENA_H();

		if (regv[3] & 0x80) {
			// transceiver 2 power down
			PIN_XCVR2_SHDNB_L();
			PIN_XCVR2_RXENA_L();
			PIN_XCVR2_TXENA_L();
			pd2 = 1; // keep track of power down
		} else {
			// transceiver 2 in standby:  no tx/rx, just synthesizer
			PIN_XCVR2_RXENA_L();
			PIN_XCVR2_TXENA_L();
			PIN_XCVR2_SHDNB_H();
			pd2 = 0; // keep track of power down

			// Standby register 2
			//	max2829_reg(0x3007,2,2);	// MIMO applications????
			max2829_reg(0x1007, 2, 2); // Normal operation
			// frequency
			program_tuner_pll(2);
			// bandwidths
			bandwidth(2);
			// Rx Control/RSSI Register
			// Rx VGA gain programmed with serial data bits in the Rx gain register (D6:D0).
			// RSSI range 0.5-2.5V
			// RSSI always enabled, 100 Hz AC coupling -3dB cutoff frequency
			max2829_reg(0x1C21, 8, 2);
			// Tx Linearity/Baseband Gain Register
			// Enable Tx VGA Gain Programming Serially
			// maximum Tx VGA Linearity
			// maximum Tx Upconverter Linearity
			// maximum Tx Baseband Gain
			max2829_reg(0x04CF, 9, 2);
		}
	}
}

void max2829_reg(int regval, int addr, int cs) {
	// 3-wire serial interface  14-bit data, 4-bit address, 2 bits for chip select
	// See Figure 2. 3-Wire Serial-Interface Timing Diagram
	// when cs(0) = 1, send data to first max2829
	// when cs(1) = 1, send data to second max2829.
	// To write the same data/reg to both max2829's, enable both cs at the same time.

	int i;
	//	PIN_XCVR_SCLK_L();
	// PIN_XCVR_SCLK_H();
	// PIN_XCVR_SCLK_L();

	// enable one or both serial interfaces
	if (cs & 0x01)
		PIN_XCVR1_CSB_L();
	if (cs & 0x02)
		PIN_XCVR2_CSB_L();

	for (i = 0; i < 14; i++) {
		if (regval & 0x2000) {
			PIN_XCVR_DIN_H(); //data high
		} else {
			PIN_XCVR_DIN_L(); //data low
		}
		PIN_XCVR_SCLK_H();
		PIN_XCVR_SCLK_L(); //clock low
		regval <<= 1;
	}
	for (i = 0; i < 4; i++) {
		if (addr & 0x0008) {
			PIN_XCVR_DIN_H(); //addr high
		} else {
			PIN_XCVR_DIN_L(); //addr low
		}
		PIN_XCVR_SCLK_H();
		PIN_XCVR_SCLK_L(); //clock low
		addr <<= 1;
	}

	PIN_XCVR1_CSB_H();
	PIN_XCVR2_CSB_H();
	PIN_XCVR_SCLK_H();
	PIN_XCVR_SCLK_L();

}

void rx_txn(void) {
	// a = 1 for receive, a = 0 for transmit
	// Fast toggling under the control of the connector RX_TXNx signal
	// poll frequently

	// transceiver 1
	if (~(regv[1] & 0x0C)) {
		// rx_txn fast switching under the control of the external RX_TXN1 signal
		// did RX_TXN1 change? if not, no need to waste time configuring all these control pins
		if(rx_txn1 != (FIO0PIN & 0x00008000)){
			//RX_TXN1 just changed
			rx_txn1 = FIO0PIN & 0x00008000;	// remember the new RX_TXN1 = P0[15]

			if (rx_txn1) {
				// receive mode
				PIN_XCVR1_TXENA_L();
				PIN_XCVR1_RXENA_H();
				PIN_TX0AG_L();
				PIN_PU0A_L();
				PIN_PU0G_L();
				PIN_RX0AG_H();
			} else {
				// transmit mode
				PIN_XCVR1_RXENA_L();
				PIN_XCVR1_TXENA_H();
				PIN_RX0AG_L();
				PIN_TX0AG_H();
				if(freqBand1){
					// 5 GHz band
					PIN_PU0G_L();
					PIN_PU0A_H();
				} else {
					// 2.4 GHz band
					PIN_PU0G_H();
					PIN_PU0A_L();
				}
			}
		}
	} else if (regv[1] & 0x04) {
		// receive-only. ignore RX_TXN1
		PIN_XCVR1_TXENA_L();
		PIN_XCVR1_RXENA_H();
		PIN_TX0AG_L();
		PIN_PU0A_L();
		PIN_PU0G_L();
		PIN_RX0AG_H();
	} else if (regv[1] & 0x08) {
		// transmit-only. ignore RX_TXN1
		PIN_XCVR1_RXENA_L();
		PIN_XCVR1_TXENA_H();
		PIN_RX0AG_L();
		PIN_TX0AG_H();
		if(freqBand1){
			// 5 GHz band
			PIN_PU0G_L();
			PIN_PU0A_H();
		} else {
			// 2.4 GHz band
			PIN_PU0G_H();
			PIN_PU0A_L();
		}
	}

	// transceiver 2
	if (~(regv[4] & 0x0C)) {
		// rx_txn fast switching under the control of the external RX_TXN2 signal
		// did RX_TXN2 change? if not, no need to waste time configuring all these control pins
		if(rx_txn2 != (FIO0PIN & 0x00010000)){
			//RX_TXN2 just changed
			rx_txn2 = FIO0PIN & 0x00010000;	// remember the new RX_TXN2 = P0[16]

			if (rx_txn2) {
				// receive mode
				PIN_XCVR2_TXENA_L();
				PIN_XCVR2_RXENA_H();
				PIN_TX1AG_L();
				PIN_PU1A_L();
				PIN_PU1G_L();
				PIN_RX1AG_H();
			} else {
				// transmit mode
				PIN_XCVR2_RXENA_L();
				PIN_XCVR2_TXENA_H();
				PIN_RX1AG_L();
				PIN_TX1AG_H();
				if(freqBand2){
					// 5 GHz band
					PIN_PU1G_L();
					PIN_PU1A_H();
				} else {
					// 2.4 GHz band
					PIN_PU1G_H();
					PIN_PU1A_L();
				}
			}
		}
	} else if (regv[4] & 0x04) {
		// receive-only. ignore RX_TXN2
		PIN_XCVR2_TXENA_L();
		PIN_XCVR2_RXENA_H();
		PIN_TX1AG_L();
		PIN_PU1A_L();
		PIN_PU1G_L();
		PIN_RX1AG_H();
	} else if (regv[4] & 0x08) {
		// transmit-only. ignore RX_TXN2
		PIN_XCVR2_RXENA_L();
		PIN_XCVR2_TXENA_H();
		PIN_RX1AG_L();
		PIN_TX1AG_H();
		if(freqBand2){
			// 5 GHz band
			PIN_PU1G_L();
			PIN_PU1A_H();
		} else {
			// 2.4 GHz band
			PIN_PU1G_H();
			PIN_PU1A_L();
		}
	}
}

long rf_synth_lock_detect(void) {
	// bit 0: rf synth1 is locked (1) or not (0)
	// bit 1: rf synth2 is locked (1) or not (0)
	// returns 3 when both rf synthesizers are locked
	return FIO1PIN & 0x00000003; // pins P1[1] & P1[0]
}

unsigned long get_freq(long index) {
	// reconstruct the RF frequency synthesizer frequency from the control registers.
	// index range 0 - 7
	// Returns the frequency expressed in integer multiple of 2Hz
	long reg_no;
	unsigned long freq;
	long i;

	// points to control register with MSB
	if ((index < 8) && (index >= 0))
		reg_no = 11 + 4 * index;
	else
		return 0; // invalid index

	for (i = 0; i < 4; i++) {
		freq <<= 8;
		freq &= 0xFFFFFF00;
		freq |= ((unsigned long) regv[reg_no--]) & 0x000000FF;
	}
	return freq;
}

void program_tuner_pll(int ICno) {
	// program RF synthesizer for ICno 1 or 2
	// enable the correct LNA (2.4 or 5GHz). Note: LNAs stay on even during tx. slow switching.

	unsigned long regval, reg5, fi;
	double f;
	long nint, nfrac;
	int cs;

	// default frequency index for ICno is in regv[0] or regv[3] bits2:0
	if (ICno == 1) {
		fi = regv[0] & 0x07;
		cs = 0x01;
	} else if (ICno == 2) {
		fi = regv[3] & 0x07;
		cs = 0x02;
	} else {
		return; // invalid ICno
	}
	// construct frequency word (in Hz) from control registers
	f = get_freq(fi); // frequency in Hz. Note 32-bit registers are expressed in
	f = f * 2;
	// integer multiple of 2Hz units
	// band select and PLL register
	// Reference-Divider Ratio R = 1 (20 MHz)
	// D8 = 0 to enable the automatic VCO sub-band selection by the FSM
	//reg5 = 0x3822;	// MIMO applications???
	reg5 = 0x1822; // Normal operation

	// WATCH OUT FOR long OVERFLOW during the calculations below.
	// 2.4 or 5 GHz band?
	if (f < 3000000000L) {
		// 2.4 - 2.5 GHz band
		nint = f / 15000000; // integer divider ratio
		nfrac = f / 25 - (nint * 600000);
		nfrac <<= 10;
		nfrac = nfrac / 9375;
		// remember the frequency band for fast tx/rx switching at the Skyworks front-end module
		if(ICno == 1){
			freqBand1 = 0;
			PIN_LNA1_52_L();
			PIN_LNA1_24_H();
		} else if (ICno == 2){
			freqBand2 = 0;
			PIN_LNA2_52_L();
			PIN_LNA2_24_H();
		}
	} else {
		// 4.9 - 5.875 GHz band
		nint = f / 25000000; // integer divider ratio
		nfrac = f / 25 - (nint * 1000000);
		nfrac <<= 10;
		nfrac = nfrac / 15625;
		// remember the frequency band for fast tx/rx switching at the Skyworks front-end module
		if(ICno == 1){
			freqBand1 = 1;
			PIN_LNA1_24_L();
			PIN_LNA1_52_H();
		} else if (ICno == 2){
			freqBand2 = 1;
			PIN_LNA2_24_L();
			PIN_LNA2_52_H();
		}

		reg5 |= 0x0001; // 5 GHz band
		if (f > 5410000000) {
			reg5 |= 0x0040; // 5.47GHz to 5.875GHz Band
		}
	}

	// program Integer-Divider Ratio Register
	regval = (nint & 0x00FF) | ((nfrac << 12) & 0x3000);
	max2829_reg(regval, 3, cs);
	// Fractional-Divider Ratio Register
	regval = ((nfrac >> 2) & 0x03FFF);
	max2829_reg(regval, 4, cs);
	// band select and PLL register
	// enable and/or reset FSM (D7 = 0, D8 = 0)
	max2829_reg(reg5, 5, cs);
	// start the FSM D7 = 1
	reg5 |= 0x00080;
	max2829_reg(reg5, 5, cs);
	// wait 300us + margin
	delay_ns(500000);

}

//void RF_synthesizer_output_enable(long en) {
//	// enable/disable RF synthesizer output
//	if (en) {
////		PIN_SYNTH_ENABLE_H();
//	} else {
////		PIN_SYNTH_ENABLE_L();
//	}
//}

void power_up_down1(void) {
	// power down or up transceiver 1
	if (regv[0] & 0x80) {
		// power down transceiver 1
		PIN_XCVR1_SHDNB_L();
		PIN_XCVR1_RXENA_L();
		PIN_XCVR1_TXENA_L();
		pd1 = 1; // keep track of power down
	} else if (pd1) {
		// power up (and configure) transceiver 1 but only if previously powered down
		max2829_init(1);
	}
}
void power_up_down2(void) {
	// power down or up transceiver 2
	if (regv[3] & 0x80) {
		// power down transceiver 2
		PIN_XCVR2_SHDNB_L();
		PIN_XCVR2_RXENA_L();
		PIN_XCVR2_TXENA_L();
		pd2 = 1; // keep track of power down
	} else if (pd2) {
		// power up (and configure) transceiver 2 but only if previously powered down
		max2829_init(2);
	}
}

void bandwidth(int ICno) {
	// configures the tx/rx low-pass filters and RSSI filter bandwidth for ICno 1 or 2
	unsigned long regval;
	regval = 0;
	if (ICno == 1) {
		// Rx LPF Corner Frequency Coarse Adjustment
		regval = (regv[1] & 0x0030) >> 1;
		// Rx LPF Corner Frequency Fine Adjustment (Relative to the Course Setting)
		regval |= 0x0002;
		// Tx LPF Corner Frequency Coarse Adjustment
		regval |= ((regv[1] & 0x00C0) >> 1) + 0x0020; // slight change in definition (1 off)
		// 2 MHz RSSI bandwidth
	} else if (ICno == 2) {
		// Rx LPF Corner Frequency Coarse Adjustment
		regval = (regv[4] & 0x0030) >> 1;
		// Rx LPF Corner Frequency Fine Adjustment (Relative to the Course Setting)
		regval |= 0x0002;
		// Tx LPF Corner Frequency Coarse Adjustment
		regval |= ((regv[4] & 0x00C0) >> 1) + 0x0020; // slight change in definition (1 off)
		// 2 MHz RSSI bandwidth
	}
	max2829_reg(regval, 7, ICno); // apply to transceiver x
}

void DAC_init(void) {
	// initialize internal 10-bit DAC
	// 1. to enable the DAC, output must be selected to appear on the related pin p0.26
	PINSEL1 = (PINSEL1 & 0xFFCFFFFF) | 0x00200000;

	// 2. peripheral clock selection for DAC.
	// select the processor clock CCLK/4 (default)
	PCLKSEL1 = PCLKSEL1 & 0xFF3FFFFF;

	// 3. enable double buffering, no interrupts, 1us refresh
	DACCTRL = 0x00000006;
	DACCNTVAL = 30; // 1us @120MHz/4
}

void DAC(long value) {
	// output is value * Vrefp / 1024
	// 1us settling time -> 1 MHz maximum update rate
	DACR = (value << 6) & 0x0000FFC0;
}

void analogTP_poll(void){
	// converts one of 8 signals to analog test point for monitoring purposes
	switch(regv[6] & 0x0F){
	case 0:
		return;	// analog test point is disabled
	case 1:
		DAC(rx1_rssi >> 2);
		break;
	case 2:
		DAC(rx2_rssi >> 2);
		break;
	case 3:
		DAC(rx1_agc >> 2);
		break;
	case 4:
		DAC(rx2_agc >> 2);
		break;
	case 5:
		DAC(tx1_gain_ctrl >> 2);
		break;
	case 6:
		DAC(tx2_gain_ctrl >> 2);
		break;
	case 7:
		DAC(tx1_power_det >> 2);
		break;
	case 8:
		DAC(tx2_power_det >> 2);
		break;
	default:
		return;	// analog test point is disabled
	}
}

void setRxGain(int gain_dB, int cs){
	// set receive gain to either or both transceivers.
	// gain is set through max2829 Rx Gain Register (11)
	int regval, addr, i;

	// convert dBs to MAX2829 control register values
	// look-up table
	gain_dB &= 0x007F;
	if(cs & 0x01){	// transceiver 1
		if(freqBand1 == 0)
			regval = gainTable5GHz[gain_dB];
		else
			regval = gainTable2p4GHz[gain_dB];
	} else if(cs & 0x02){	// transceiver 2
		if(freqBand2 == 0)
			regval = gainTable5GHz[gain_dB];
		else
			regval = gainTable2p4GHz[gain_dB];
	} else {
		return;
	}
	addr = 11;

	// instead of using the general purpose max2829_reg function, we use code optimized for speed
	// "For faster Rx gain setting, only D6:D0 need to be programmed."
	// enable one or both serial interfaces
	if (cs & 0x01)
		PIN_XCVR1_CSB_L();
	if (cs & 0x02)
		PIN_XCVR2_CSB_L();

	for (i = 0; i < 7; i++) {
		if (regval & 0x040) {
			PIN_XCVR_DIN_H(); //data high
		} else {
			PIN_XCVR_DIN_L(); //data low
		}
		PIN_XCVR_SCLK_H();
		PIN_XCVR_SCLK_L(); //clock low
		regval <<= 1;
	}
	for (i = 0; i < 4; i++) {
		if (addr & 0x0008) {
			PIN_XCVR_DIN_H(); //addr high
		} else {
			PIN_XCVR_DIN_L(); //addr low
		}
		PIN_XCVR_SCLK_H();
		PIN_XCVR_SCLK_L(); //clock low
		addr <<= 1;
	}

	PIN_XCVR1_CSB_H();
	PIN_XCVR2_CSB_H();
	PIN_XCVR_SCLK_H();
	PIN_XCVR_SCLK_L();
}

void setTxGain(int cs){
	// set tx gain to either or both transceivers.
	// gain is set through TX1_GAIN_CTRL analog input converted to tx1/2_gain_ctrl
	int regval, addr, i;

	addr = 12;	// Tx VGA Gain Register

	// instead of using the general purpose max2829_reg function, we use code optimized for speed
	// "For faster Rx gain setting, only D6:D0 need to be programmed."
	// enable one or both serial interfaces
	if (cs & 0x01){
		PIN_XCVR1_CSB_L();
		regval = tx1_gain_ctrl >> 6;	// reduce precision from 12 to 6 bits
	} else {
		regval = tx2_gain_ctrl >> 6;	// reduce precision from 12 to 6 bits
	}
	if (cs & 0x02){
		PIN_XCVR2_CSB_L();	// enable one or both transceivers
	}
	for (i = 0; i < 6; i++) {
		if (regval & 0x020) {
			PIN_XCVR_DIN_H(); //data high
		} else {
			PIN_XCVR_DIN_L(); //data low
		}
		PIN_XCVR_SCLK_H();
		PIN_XCVR_SCLK_L(); //clock low
		regval <<= 1;
	}
	for (i = 0; i < 4; i++) {
		if (addr & 0x0008) {
			PIN_XCVR_DIN_H(); //addr high
		} else {
			PIN_XCVR_DIN_L(); //addr low
		}
		PIN_XCVR_SCLK_H();
		PIN_XCVR_SCLK_L(); //clock low
		addr <<= 1;
	}

	PIN_XCVR1_CSB_H();
	PIN_XCVR2_CSB_H();
	PIN_XCVR_SCLK_H();
	PIN_XCVR_SCLK_L();
}

void RxAGC_poll(int cs) {
	// receive AGC

	// transceiver 1
	if(cs & 0x01){
		if((regv[1] & 0x0003) == 1){
			// local AGC loop (unused)

		} else if ((regv[1] & 0x0003) == 2){
			// external AGC loop
			// inverted scale 0 = maximum gain, 3.3V = minimum gain
			RF_gain1 = 95 - ((rx1_agc * 95)/4096);
			setRxGain(RF_gain1, 1);	// set initial RF gain
		} else {
			// fixed rx gain.  already set in main.c
		}
	}

	// transceiver 2
	if(cs & 0x02){
		if((regv[4] & 0x0003) == 1){
			// local AGC loop (unused)

		} else if ((regv[4] & 0x0003) == 2){
			// external AGC loop
			// inverted scale 0 = maximum gain, 3.3V = minimum gain
			RF_gain2 = 95 - ((rx2_agc * 95)/4096);
			setRxGain(RF_gain2, 2);	// set initial RF gain
		} else {
			// fixed rx gain.  already set in main.c
		}
	}
}

void ADC_init(void) {
	// ADC initialization
	PCONP |= 0x00001000; // power ADC subsystem
	// PCLKSEL0: by default peripheral clock is CCLK/4 = 30 MHz
	// A/D Control Register: input is only AD0[5], ADC clock is 10 MHz, burst=0 (i.e no continuous ADC), operational,
	AD0CR = 0x00200220;
	// initialize the adc mux to select RX1_RSSI
	adc_mux = 0;
	// start conversion now
	AD0CR = 0x01200220;
	// wait for conversion completion (next poll)
}

void ADC_poll() {
	// Conduct measurements on rx or tx analog controls and monitoring signals.
	// During rx and while external AGC, successively and periodically measure RX1_AGC, RX2_AGC
	// During rx and while internal AGC, successively and periodically measure RX1_RSSI, RX2_RSSI
	// During tx, successively and periodically measure TX1_GAIN_CTRL, TX2_GAIN_CTRL, TX1_POWER_DET, TX2_POWER_DET
	unsigned long a;

	a = AD0GDR;
	if(!(a & 0x80000000)) // ADC still converting. try again later
		return;

	// ADC done with last conversion. Save it
	a = (a >> 4) & 0x00000FFF; // 12-bit ADC measurement (result bits 15:4)
	switch (adc_mux) {
	case 0:
		rx1_rssi = a;
		break;
	case 1:
		rx2_rssi = a;
		break;
	case 2:
		rx1_agc = a;
		break;
	case 3:
		rx2_agc = a;
		break;
	case 4:
		tx1_gain_ctrl = a;
		break;
	case 5:
		tx2_gain_ctrl = a;
		break;
	case 6:
		tx1_power_det = a;
		break;
	default:
		tx2_power_det = a;
		break;
	}

	// select the next signal to be sampled and quantized by the ADC.
	adc_mux++;
	if(adc_mux > 7)
		adc_mux = 0;	// round robbin 0 - 7 and back to zero.
	if((adc_mux == 0) && (!rx_txn1)){
		// transmitting. skip rx1_rssi measurement
		adc_mux++;
	}
	if((adc_mux == 1) && (!rx_txn2)){
		// transmitting. skip rx2_rssi measurement
		adc_mux++;
	}
	if((adc_mux == 2) && (!rx_txn1)){
		// transmitting. skip rx1_agc measurement
		adc_mux++;
	}
	if((adc_mux == 3) && (!rx_txn2)){
		// transmitting. skip rx2_agc measurement
		adc_mux++;
	}
	if((adc_mux == 4) && rx_txn1){
		// receiving. skip tx1_gain_ctrl measurement
		adc_mux++;
	}
	if((adc_mux == 5) && rx_txn2){
		// receiving. skip tx2_gain_ctrl measurement
		adc_mux++;
	}
	if((adc_mux == 6) && rx_txn1){
		// receiving. skip tx1_power_det measurement
		adc_mux++;
	}
	if((adc_mux == 7) && rx_txn2){
		// receiving. skip tx2_power_det measurement
		adc_mux = 0;
	}

	// mux selection, P1[26/28/29]
	if (adc_mux & 0x0001)
		FIO1PIN = (FIO1PIN & ~0x04000000) | 0x04000000;
	else
		FIO1PIN = (FIO1PIN & ~0x04000000);
	if (adc_mux & 0x0002)
		FIO1PIN = (FIO1PIN & ~0x10000000) | 0x10000000;
	else
		FIO1PIN = (FIO1PIN & ~0x10000000);
	if (adc_mux & 0x0004)
		FIO1PIN = (FIO1PIN & ~0x20000000) | 0x20000000;
	else
		FIO1PIN = (FIO1PIN & ~0x20000000);
	// start conversion now
	AD0CR = 0x01200220;
	// wait for conversion completion (next poll)

	toggle2(1);

}


//void ADC(int mux) {
//	// returns the last value read on AD0[5] for the selected mux value. Mux value in the range 0-7
//	// 0 = RX1_RSSI,  1 = RX2_RSSI, 2 = RX1_AGC, 3 = RX2_AGC, 4 = TX1_GAIN_CTRL
//	// 5 = TX2_GAIN_CTRL, 6 = TX1_POWER_DET, 7 = TX2_POWER_DET
//	unsigned long a;
//
//	if ((mux < 0) || (mux > 7))
//		return; // mux value out of range
//	// mux selection, P1[26/28/29]
//	if (mux & 0x0001)
//		FIO1PIN = (FIO1PIN & ~0x04000000) | 0x04000000;
//	else
//		FIO1PIN = (FIO1PIN & ~0x04000000);
//	if (mux & 0x0002)
//		FIO1PIN = (FIO1PIN & ~0x10000000) | 0x10000000;
//	else
//		FIO1PIN = (FIO1PIN & ~0x10000000);
//	if (mux & 0x0004)
//		FIO1PIN = (FIO1PIN & ~0x20000000) | 0x20000000;
//	else
//		FIO1PIN = (FIO1PIN & ~0x20000000);
//	// start conversion now
//	AD0CR = 0x01200220;
//	// wait for conversion completion
//	while (1) {
//		a = AD0GDR;
//		if (a & 0x80000000) // DONE bit high
//			break;
//	}
//
//	a = (a >> 4) & 0x00000FFF; // 12-bit ADC measurement (result bits 15:4)
//	switch (mux) {
//	case 0:
//		rx1_rssi = a;
//		break;
//	case 1:
//		rx2_rssi = a;
//		break;
//	case 2:
//		rx1_agc = a;
//		break;
//	case 3:
//		rx2_agc = a;
//		break;
//	case 4:
//		tx1_gain_ctrl = a;
//		break;
//	case 5:
//		tx2_gain_ctrl = a;
//		break;
//	case 6:
//		tx1_power_det = a;
//		break;
//	default:
//		tx2_power_det = a;
//		break;
//	}
//}

